"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHubRelease = void 0;
const chalk = require("chalk");
const checkpoint_1 = require("./util/checkpoint");
const release_pr_factory_1 = require("./release-pr-factory");
const github_1 = require("./github");
const semver_1 = require("semver");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const parseGithubRepoUrl = require('parse-github-repo-url');
const GITHUB_RELEASE_LABEL = 'autorelease: tagged';
class GitHubRelease {
    constructor(options) {
        var _a;
        this.apiUrl = options.apiUrl;
        this.proxyKey = options.proxyKey;
        this.labels = options.label.split(',');
        this.repoUrl = options.repoUrl;
        this.monorepoTags = options.monorepoTags;
        this.token = options.token;
        this.path = options.path;
        this.packageName = options.packageName;
        this.releaseType = options.releaseType;
        this.changelogPath = (_a = options.changelogPath) !== null && _a !== void 0 ? _a : 'CHANGELOG.md';
        this.gh = this.gitHubInstance(options.octokitAPIs);
    }
    async createRelease() {
        // In most configurations, createRelease() should be called close to when
        // a release PR is merged, e.g., a GitHub action that kicks off this
        // workflow on merge. For tis reason, we can pull a fairly small number of PRs:
        const pageSize = 25;
        const gitHubReleasePR = await this.gh.findMergedReleasePR(this.labels, pageSize, this.monorepoTags ? this.packageName : undefined);
        if (!gitHubReleasePR) {
            checkpoint_1.checkpoint('no recent release PRs found', checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const version = `v${gitHubReleasePR.version}`;
        checkpoint_1.checkpoint(`found release branch ${chalk.green(version)} at ${chalk.green(gitHubReleasePR.sha)}`, checkpoint_1.CheckpointType.Success);
        const changelogContents = (await this.gh.getFileContents(this.addPath(this.changelogPath))).parsedContent;
        const latestReleaseNotes = GitHubRelease.extractLatestReleaseNotes(changelogContents, version);
        checkpoint_1.checkpoint(`found release notes: \n---\n${chalk.grey(latestReleaseNotes)}\n---\n`, checkpoint_1.CheckpointType.Success);
        // Attempt to lookup the package name from a well known location, such
        // as package.json, if none is provided:
        if (!this.packageName && this.releaseType) {
            this.packageName = await release_pr_factory_1.ReleasePRFactory.class(this.releaseType).lookupPackageName(this.gh);
        }
        // Go uses '/' for a tag separator, rather than '-':
        let tagSeparator = '-';
        if (this.releaseType) {
            tagSeparator = release_pr_factory_1.ReleasePRFactory.class(this.releaseType).tagSeparator();
        }
        if (this.packageName === undefined) {
            throw Error(`could not determine package name for release repo = ${this.repoUrl}`);
        }
        const release = await this.gh.createRelease(this.packageName, this.monorepoTags
            ? `${this.packageName}${tagSeparator}${version}`
            : version, gitHubReleasePR.sha, latestReleaseNotes);
        // Add a label indicating that a release has been created on GitHub,
        // but a publication has not yet occurred.
        await this.gh.addLabels([GITHUB_RELEASE_LABEL], gitHubReleasePR.number);
        // Remove 'autorelease: pending' which indicates a GitHub release
        // has not yet been created.
        await this.gh.removeLabels(this.labels, gitHubReleasePR.number);
        const parsedVersion = semver_1.parse(version, { loose: true });
        if (parsedVersion) {
            return {
                major: parsedVersion.major,
                minor: parsedVersion.minor,
                patch: parsedVersion.patch,
                sha: gitHubReleasePR.sha,
                version,
                pr: gitHubReleasePR.number,
                html_url: release.html_url,
                tag_name: release.tag_name,
                upload_url: release.upload_url,
            };
        }
        else {
            console.warn(`failed to parse version informatino from ${version}`);
            return undefined;
        }
    }
    addPath(file) {
        if (this.path === undefined) {
            return file;
        }
        else {
            const path = this.path.replace(/[/\\]$/, '');
            file = file.replace(/^[/\\]/, '');
            return `${path}/${file}`;
        }
    }
    gitHubInstance(octokitAPIs) {
        const [owner, repo] = parseGithubRepoUrl(this.repoUrl);
        return new github_1.GitHub({
            token: this.token,
            owner,
            repo,
            apiUrl: this.apiUrl,
            proxyKey: this.proxyKey,
            octokitAPIs,
        });
    }
    static extractLatestReleaseNotes(changelogContents, version) {
        version = version.replace(/^v/, '');
        const latestRe = new RegExp(`## v?\\[?${version}[^\\n]*\\n(.*?)(\\n##\\s|\\n### \\[?[0-9]+\\.|($(?![\r\n])))`, 'ms');
        const match = changelogContents.match(latestRe);
        if (!match) {
            throw Error('could not find changelog entry corresponding to release PR');
        }
        return match[1];
    }
}
exports.GitHubRelease = GitHubRelease;
//# sourceMappingURL=github-release.js.map