"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHub = void 0;
const code_suggester_1 = require("code-suggester");
const rest_1 = require("@octokit/rest");
const request_1 = require("@octokit/request");
const graphql_1 = require("@octokit/graphql");
const chalk = require("chalk");
const semver = require("semver");
const checkpoint_1 = require("./util/checkpoint");
const graphql_to_commits_1 = require("./graphql-to-commits");
// Short explanation of this regex:
// - skip the owner tag (e.g. googleapis)
// - make sure the branch name starts with "release"
// - take everything else
// This includes the tag to handle monorepos.
const VERSION_FROM_BRANCH_RE = /^.*:release-?([\w-.]*)-(v[0-9].*)$/;
let probotMode = false;
class GitHub {
    constructor(options) {
        this.defaultBranch = options.defaultBranch;
        this.token = options.token;
        this.owner = options.owner;
        this.repo = options.repo;
        this.apiUrl = options.apiUrl || 'https://api.github.com';
        this.proxyKey = options.proxyKey;
        if (options.octokitAPIs === undefined) {
            this.octokit = new rest_1.Octokit({
                baseUrl: options.apiUrl,
                auth: this.token,
            });
            const defaults = {
                baseUrl: this.apiUrl,
                headers: {
                    'user-agent': `release-please/${require('../../package.json').version}`,
                    // some proxies do not require the token prefix.
                    Authorization: `${this.proxyKey ? '' : 'token '}${this.token}`,
                },
            };
            this.request = request_1.request.defaults(defaults);
            this.graphql = graphql_1.graphql;
        }
        else {
            // for the benefit of probot applications, we allow a configured instance
            // of octokit to be passed in as a parameter.
            probotMode = true;
            this.octokit = options.octokitAPIs.octokit;
            this.request = options.octokitAPIs.request;
            this.graphql = options.octokitAPIs.graphql;
        }
    }
    async graphqlRequest(_opts) {
        let opts = Object.assign({}, _opts);
        if (!probotMode) {
            opts = Object.assign(opts, {
                url: `${this.apiUrl}/graphql${this.proxyKey ? `?key=${this.proxyKey}` : ''}`,
                headers: {
                    authorization: `${this.proxyKey ? '' : 'token '}${this.token}`,
                    'content-type': 'application/vnd.github.v3+json',
                },
            });
        }
        return this.graphql(opts);
    }
    decoratePaginateOpts(opts) {
        if (probotMode) {
            return opts;
        }
        else {
            return Object.assign(opts, {
                headers: {
                    Authorization: `${this.proxyKey ? '' : 'token '}${this.token}`,
                },
            });
        }
    }
    async commitsSinceSha(sha, perPage = 100, labels = false, path = null) {
        const commits = [];
        const method = labels ? 'commitsWithLabels' : 'commitsWithFiles';
        let cursor;
        for (;;) {
            const commitsResponse = await this[method](cursor, perPage, path);
            for (let i = 0, commit; i < commitsResponse.commits.length; i++) {
                commit = commitsResponse.commits[i];
                if (commit.sha === sha) {
                    return commits;
                }
                else {
                    commits.push(commit);
                }
            }
            if (commitsResponse.hasNextPage === false || !commitsResponse.endCursor) {
                return commits;
            }
            else {
                cursor = commitsResponse.endCursor;
            }
        }
    }
    async commitsWithFiles(cursor = undefined, perPage = 32, path = null, maxFilesChanged = 64, retries = 0) {
        const baseBranch = await this.getDefaultBranch();
        // The GitHub v3 API does not offer an elegant way to fetch commits
        // in conjucntion with the path that they modify. We lean on the graphql
        // API for this one task, fetching commits in descending chronological
        // order along with the file paths attached to them.
        try {
            const response = await this.graphqlRequest({
                query: `query commitsWithFiles($cursor: String, $owner: String!, $repo: String!, $baseRef: String!, $perPage: Int, $maxFilesChanged: Int, $path: String) {
          repository(owner: $owner, name: $repo) {
            ref(qualifiedName: $baseRef) {
              target {
                ... on Commit {
                  history(first: $perPage, after: $cursor, path: $path) {
                    edges {
                      node {
                        ... on Commit {
                          message
                          oid
                          associatedPullRequests(first: 1) {
                            edges {
                              node {
                                ... on PullRequest {
                                  number
                                  mergeCommit {
                                    oid
                                  }
                                  files(first: $maxFilesChanged) {
                                    edges {
                                      node {
                                        path
                                      }
                                    }
                                    pageInfo {
                                      endCursor
                                      hasNextPage
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                    pageInfo {
                      endCursor
                      hasNextPage
                    }
                  }
                }
              }
            }
          }
        }`,
                cursor,
                maxFilesChanged,
                owner: this.owner,
                path,
                perPage,
                repo: this.repo,
                baseRef: `refs/heads/${baseBranch}`,
            });
            return graphql_to_commits_1.graphqlToCommits(this, response);
        }
        catch (err) {
            if (err.status === 502 && retries < 3) {
                // GraphQL sometimes returns a 502 on the first request,
                // this seems to relate to a cache being warmed and the
                // second request generally works.
                return this.commitsWithFiles(cursor, perPage, path, maxFilesChanged, retries++);
            }
            else {
                throw err;
            }
        }
    }
    async commitsWithLabels(cursor = undefined, perPage = 32, path = null, maxLabels = 16, retries = 0) {
        const baseBranch = await this.getDefaultBranch();
        try {
            const response = await this.graphqlRequest({
                query: `query commitsWithLabels($cursor: String, $owner: String!, $repo: String!, $baseRef: String!, $perPage: Int, $maxLabels: Int, $path: String) {
          repository(owner: $owner, name: $repo) {
            ref(qualifiedName: $baseRef) {
              target {
                ... on Commit {
                  history(first: $perPage, after: $cursor, path: $path) {
                    edges {
                      node {
                        ... on Commit {
                          message
                          oid
                          associatedPullRequests(first: 1) {
                            edges {
                              node {
                                ... on PullRequest {
                                  number
                                  mergeCommit {
                                    oid
                                  }
                                  labels(first: $maxLabels) {
                                    edges {
                                      node {
                                        name
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                    pageInfo {
                      endCursor
                      hasNextPage
                    }
                  }
                }
              }
            }
          }
        }`,
                cursor,
                maxLabels,
                owner: this.owner,
                path,
                perPage,
                repo: this.repo,
                baseRef: `refs/heads/${baseBranch}`,
            });
            return graphql_to_commits_1.graphqlToCommits(this, response);
        }
        catch (err) {
            if (err.status === 502 && retries < 3) {
                // GraphQL sometimes returns a 502 on the first request,
                // this seems to relate to a cache being warmed and the
                // second request generally works.
                return this.commitsWithLabels(cursor, perPage, path, maxLabels, retries++);
            }
            else {
                throw err;
            }
        }
    }
    async pullRequestFiles(num, cursor, maxFilesChanged = 100) {
        // Used to handle the edge-case in which a PR has more than 100
        // modified files attached to it.
        const response = await this.graphqlRequest({
            query: `query pullRequestFiles($cursor: String, $owner: String!, $repo: String!, $maxFilesChanged: Int, $num: Int!) {
          repository(owner: $owner, name: $repo) {
            pullRequest(number: $num) {
              number
              files(first: $maxFilesChanged, after: $cursor) {
                edges {
                  node {
                    path
                  }
                }
                pageInfo {
                  endCursor
                  hasNextPage
                }
              }
            }
          }
        }`,
            cursor,
            maxFilesChanged,
            owner: this.owner,
            repo: this.repo,
            num,
        });
        return { node: response.repository.pullRequest };
    }
    async getTagSha(name) {
        const refResponse = (await this.request(`GET /repos/:owner/:repo/git/refs/tags/:name${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
            owner: this.owner,
            repo: this.repo,
            name,
        }));
        return refResponse.data.object.sha;
    }
    // This looks for the most recent matching release tag on
    // the branch we're configured for.
    async latestTag(prefix, preRelease = false, 
    // Allow a branch prefix to differ from a tag prefix. This was required
    // for google-cloud-go, which uses the tag prefix library/vx.y.z.
    // this is a requirement in the go community.
    branchPrefix) {
        const pull = await this.findMergedReleasePR([], 100, branchPrefix !== null && branchPrefix !== void 0 ? branchPrefix : prefix, preRelease);
        if (!pull)
            return await this.latestTagFallback(prefix, preRelease);
        const tag = {
            name: `v${pull.version}`,
            sha: pull.sha,
            version: pull.version,
        };
        return tag;
    }
    // If we can't find a release branch (a common cause of this, as an example
    // is that we might be dealing with the first relese), use the last semver
    // tag that's available on the repository:
    // TODO: it would be good to not need to maintain this logic, and the
    // logic that introspects version based on the prior release PR.
    async latestTagFallback(prefix, preRelease = false) {
        const tags = await this.allTags(prefix);
        const versions = Object.keys(tags).filter(t => {
            // remove any pre-releases from the list:
            return preRelease || !t.includes('-');
        });
        // no tags have been created yet.
        if (versions.length === 0)
            return undefined;
        // We use a slightly modified version of semver's sorting algorithm, which
        // prefixes the numeric part of a pre-release with '0's, so that
        // 010 is greater than > 002.
        versions.sort((v1, v2) => {
            if (v1.includes('-')) {
                const [prefix, suffix] = v1.split('-');
                v1 = prefix + '-' + suffix.replace(/[a-zA-Z.]/, '').padStart(6, '0');
            }
            if (v2.includes('-')) {
                const [prefix, suffix] = v2.split('-');
                v2 = prefix + '-' + suffix.replace(/[a-zA-Z.]/, '').padStart(6, '0');
            }
            return semver.rcompare(v1, v2);
        });
        return {
            name: tags[versions[0]].name,
            sha: tags[versions[0]].sha,
            version: tags[versions[0]].version,
        };
    }
    async allTags(prefix) {
        const tags = {};
        for await (const response of this.octokit.paginate.iterator(this.decoratePaginateOpts({
            method: 'GET',
            url: `/repos/${this.owner}/${this.repo}/tags?per_page=100${this.proxyKey ? `&key=${this.proxyKey}` : ''}`,
        }))) {
            response.data.forEach((data) => {
                // For monorepos, a prefix can be provided, indicating that only tags
                // matching the prefix should be returned:
                if (prefix && !data.name.startsWith(prefix))
                    return;
                let version = data.name.replace(prefix, '');
                if ((version = semver.valid(version))) {
                    tags[version] = { sha: data.commit.sha, name: data.name, version };
                }
            });
        }
        return tags;
    }
    // The default matcher will rule out pre-releases.
    // TODO: make this handle more than 100 results using async iterator.
    async findMergedReleasePR(labels, perPage = 100, prefix = undefined, preRelease = true) {
        prefix = (prefix === null || prefix === void 0 ? void 0 : prefix.endsWith('-')) ? prefix.replace(/-$/, '') : prefix;
        const baseLabel = await this.getBaseLabel();
        const pullsResponse = (await this.request(`GET /repos/:owner/:repo/pulls?state=closed&per_page=${perPage}${this.proxyKey ? `&key=${this.proxyKey}` : ''}&sort=merged_at&direction=desc`, {
            owner: this.owner,
            repo: this.repo,
        }));
        for (const pull of pullsResponse.data) {
            if (labels.length === 0 ||
                this.hasAllLabels(labels, pull.labels.map(l => {
                    return l.name + '';
                }))) {
                // it's expected that a release PR will have a
                // HEAD matching the format repo:release-v1.0.0.
                if (!pull.head)
                    continue;
                // Verify that this PR was based against our base branch of interest.
                if (!pull.base || pull.base.label !== baseLabel)
                    continue;
                // The input should look something like:
                // user:release-[optional-package-name]-v1.2.3
                // We want the package name and any semver on the end.
                const match = pull.head.label.match(VERSION_FROM_BRANCH_RE);
                if (!match || !pull.merged_at)
                    continue;
                // The input here should look something like:
                // [optional-package-name-]v1.2.3[-beta-or-whatever]
                // Because the package name can contain things like "-v1",
                // it's easiest/safest to just pull this out by string search.
                const version = match[2];
                if (!version)
                    continue;
                if (prefix && match[1] !== prefix) {
                    continue;
                }
                else if (!prefix && match[1]) {
                    continue;
                }
                // What's left by now should just be the version string.
                // Check for pre-releases if needed.
                if (!preRelease && version.indexOf('-') >= 0)
                    continue;
                // Make sure we did get a valid semver.
                const normalizedVersion = semver.valid(version);
                if (!normalizedVersion)
                    continue;
                return {
                    number: pull.number,
                    sha: pull.merge_commit_sha,
                    version: normalizedVersion,
                };
            }
        }
        return undefined;
    }
    hasAllLabels(labelsA, labelsB) {
        let hasAll = true;
        labelsA.forEach(label => {
            if (labelsB.indexOf(label) === -1)
                hasAll = false;
        });
        return hasAll;
    }
    async findOpenReleasePRs(labels, perPage = 100) {
        const baseLabel = await this.getBaseLabel();
        const openReleasePRs = [];
        const pullsResponse = (await this.request(`GET /repos/:owner/:repo/pulls?state=open&per_page=${perPage}${this.proxyKey ? `&key=${this.proxyKey}` : ''}`, {
            owner: this.owner,
            repo: this.repo,
        }));
        for (const pull of pullsResponse.data) {
            // Verify that this PR was based against our base branch of interest.
            if (!pull.base || pull.base.label !== baseLabel)
                continue;
            let hasAllLabels = false;
            const observedLabels = pull.labels.map(l => l.name);
            for (const expectedLabel of labels) {
                if (observedLabels.includes(expectedLabel)) {
                    hasAllLabels = true;
                }
                else {
                    hasAllLabels = false;
                    break;
                }
            }
            if (hasAllLabels)
                openReleasePRs.push(pull);
        }
        return openReleasePRs;
    }
    async addLabels(labels, pr) {
        checkpoint_1.checkpoint(`adding label ${chalk.green(labels.join(','))} to https://github.com/${this.owner}/${this.repo}/pull/${pr}`, checkpoint_1.CheckpointType.Success);
        return this.request(`POST /repos/:owner/:repo/issues/:issue_number/labels${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
            owner: this.owner,
            repo: this.repo,
            issue_number: pr,
            labels,
        });
    }
    async findExistingReleaseIssue(title, labels) {
        try {
            for await (const response of this.octokit.paginate.iterator(this.decoratePaginateOpts({
                method: 'GET',
                url: `/repos/${this.owner}/${this.repo}/issues?labels=${labels.join(',')}${this.proxyKey ? `&key=${this.proxyKey}` : ''}`,
                per_page: 100,
            }))) {
                for (let i = 0; response.data[i] !== undefined; i++) {
                    const issue = response.data[i];
                    if (issue.title.indexOf(title) !== -1 && issue.state === 'open') {
                        return issue;
                    }
                }
            }
        }
        catch (err) {
            if (err.status === 404) {
                // the most likely cause of a 404 during this step is actually
                // that the user does not have access to the repo:
                throw new AuthError();
            }
            else {
                throw err;
            }
        }
        return undefined;
    }
    async openPR(options) {
        const defaultBranch = await this.getDefaultBranch();
        // check if there's an existing PR, so that we can opt to update it
        // rather than creating a new PR.
        const refName = `refs/heads/${options.branch}`;
        let openReleasePR;
        const releasePRCandidates = await this.findOpenReleasePRs(options.labels);
        for (const releasePR of releasePRCandidates) {
            if (refName && refName.includes(releasePR.head.ref)) {
                openReleasePR = releasePR;
                break;
            }
        }
        // Short-circuit if there have been no changes to the pull-request body.
        if (openReleasePR && openReleasePR.body === options.body) {
            checkpoint_1.checkpoint(`PR https://github.com/${this.owner}/${this.repo}/pull/${openReleasePR.number} remained the same`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        //  Actually update the files for the release:
        const changes = await this.getChangeSet(options.updates, defaultBranch);
        const prNumber = await code_suggester_1.createPullRequest(this.octokit, changes, {
            upstreamOwner: this.owner,
            upstreamRepo: this.repo,
            title: options.title,
            branch: options.branch,
            description: options.body,
            primary: defaultBranch,
            force: true,
            fork: options.fork,
            message: options.title,
        }, { level: 'error' });
        // If a release PR was already open, update the title and body:
        if (openReleasePR) {
            checkpoint_1.checkpoint(`update pull-request #${openReleasePR.number}: ${chalk.yellow(options.title)}`, checkpoint_1.CheckpointType.Success);
            await this.request(`PATCH /repos/:owner/:repo/pulls/:pull_number${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
                pull_number: openReleasePR.number,
                owner: this.owner,
                repo: this.repo,
                title: options.title,
                body: options.body,
                state: 'open',
            });
            return openReleasePR.number;
        }
        else {
            return prNumber;
        }
    }
    async getChangeSet(updates, defaultBranch) {
        const changes = new Map();
        for (const update of updates) {
            let content;
            try {
                if (update.contents) {
                    // we already loaded the file contents earlier, let's not
                    // hit GitHub again.
                    content = { data: update.contents };
                }
                else {
                    const fileContent = await this.getFileContentsOnBranch(update.path, defaultBranch);
                    content = { data: fileContent };
                }
            }
            catch (err) {
                if (err.status !== 404)
                    throw err;
                // if the file is missing and create = false, just continue
                // to the next update, otherwise create the file.
                if (!update.create) {
                    checkpoint_1.checkpoint(`file ${chalk.green(update.path)} did not exist`, checkpoint_1.CheckpointType.Failure);
                    continue;
                }
            }
            const contentText = content
                ? Buffer.from(content.data.content, 'base64').toString('utf8')
                : undefined;
            const updatedContent = update.updateContent(contentText);
            if (updatedContent) {
                changes.set(update.path, {
                    content: updatedContent,
                    mode: '100644',
                });
            }
        }
        return changes;
    }
    // The base label is basically the default branch, attached to the owner.
    async getBaseLabel() {
        const baseBranch = await this.getDefaultBranch();
        return `${this.owner}:${baseBranch}`;
    }
    async getDefaultBranch() {
        if (this.defaultBranch) {
            return this.defaultBranch;
        }
        const { data } = await this.octokit.repos.get({
            repo: this.repo,
            owner: this.owner,
            headers: {
                Authorization: `${this.proxyKey ? '' : 'token '}${this.token}`,
            },
        });
        this.defaultBranch = data.default_branch;
        return this.defaultBranch;
    }
    async closePR(prNumber) {
        await this.request(`PATCH /repos/:owner/:repo/pulls/:pull_number${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
            owner: this.owner,
            repo: this.repo,
            pull_number: prNumber,
            state: 'closed',
        });
    }
    // Takes a potentially unqualified branch name, and turns it
    // into a fully qualified ref.
    //
    // e.g. main -> refs/heads/main
    static qualifyRef(refName) {
        let final = refName;
        if (final.indexOf('/') < 0) {
            final = `refs/heads/${final}`;
        }
        return final;
    }
    async getFileContentsWithSimpleAPI(path, branch) {
        const ref = GitHub.qualifyRef(branch);
        const options = {
            owner: this.owner,
            repo: this.repo,
            path,
        };
        if (ref)
            options.ref = ref;
        const resp = await this.request(`GET /repos/:owner/:repo/contents/:path${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, options);
        return {
            parsedContent: Buffer.from(resp.data.content, 'base64').toString('utf8'),
            content: resp.data.content,
            sha: resp.data.sha,
        };
    }
    async getFileContentsWithDataAPI(path, branch) {
        const options = {
            owner: this.owner,
            repo: this.repo,
            branch,
        };
        const repoTree = await this.request(`GET /repos/:owner/:repo/git/trees/:branch${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, options);
        const blobDescriptor = repoTree.data.tree.find(tree => tree.path === path);
        if (!blobDescriptor) {
            throw new Error(`Could not find requested path: ${path}`);
        }
        const resp = await this.request(`GET /repos/:owner/:repo/git/blobs/:sha${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
            owner: this.owner,
            repo: this.repo,
            sha: blobDescriptor.sha,
        });
        return {
            parsedContent: Buffer.from(resp.data.content, 'base64').toString('utf8'),
            content: resp.data.content,
            sha: resp.data.sha,
        };
    }
    async getFileContents(path) {
        return await this.getFileContentsOnBranch(path, await this.getDefaultBranch());
    }
    async getFileContentsOnBranch(path, branch) {
        try {
            return await this.getFileContentsWithSimpleAPI(path, branch);
        }
        catch (err) {
            if (err.status === 403) {
                return await this.getFileContentsWithDataAPI(path, branch);
            }
            throw err;
        }
    }
    async createRelease(packageName, tagName, sha, releaseNotes) {
        checkpoint_1.checkpoint(`creating release ${tagName}`, checkpoint_1.CheckpointType.Success);
        return (await this.request(`POST /repos/:owner/:repo/releases${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
            owner: this.owner,
            repo: this.repo,
            tag_name: tagName,
            target_commitish: sha,
            body: releaseNotes,
            name: `${packageName} ${tagName}`,
        })).data;
    }
    async removeLabels(labels, prNumber) {
        for (let i = 0, label; i < labels.length; i++) {
            label = labels[i];
            checkpoint_1.checkpoint(`removing label ${chalk.green(label)} from ${chalk.green('' + prNumber)}`, checkpoint_1.CheckpointType.Success);
            await this.request(`DELETE /repos/:owner/:repo/issues/:issue_number/labels/:name${this.proxyKey ? `?key=${this.proxyKey}` : ''}`, {
                owner: this.owner,
                repo: this.repo,
                issue_number: prNumber,
                name: label,
            });
        }
    }
    async findFilesByFilename(filename) {
        const response = await this.octokit.search.code({
            q: `filename:${filename}+repo:${this.owner}/${this.repo}`,
        });
        return response.data.items.map(file => {
            return file.path;
        });
    }
}
exports.GitHub = GitHub;
class AuthError extends Error {
    constructor() {
        super('unauthorized');
        this.status = 401;
    }
}
//# sourceMappingURL=github.js.map