"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReleasePR = void 0;
const semver = require("semver");
const checkpoint_1 = require("./util/checkpoint");
const github_1 = require("./github");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const parseGithubRepoUrl = require('parse-github-repo-url');
const DEFAULT_LABELS = 'autorelease: pending';
class ReleasePR {
    constructor(options) {
        this.bumpMinorPreMajor = options.bumpMinorPreMajor || false;
        this.defaultBranch = options.defaultBranch;
        this.fork = !!options.fork;
        this.labels = options.label
            ? options.label.split(',')
            : DEFAULT_LABELS.split(',');
        this.repoUrl = options.repoUrl;
        this.token = options.token;
        this.path = options.path;
        this.packageName = options.packageName;
        this.monorepoTags = options.monorepoTags || false;
        this.releaseAs = options.releaseAs;
        this.apiUrl = options.apiUrl;
        this.proxyKey = options.proxyKey;
        this.snapshot = options.snapshot;
        // drop a `v` prefix if provided:
        this.lastPackageVersion = options.lastPackageVersion
            ? options.lastPackageVersion.replace(/^v/, '')
            : undefined;
        this.gh = this.gitHubInstance(options.octokitAPIs);
        this.changelogSections = options.changelogSections;
    }
    async run() {
        if (this.snapshot && !this.supportsSnapshots()) {
            checkpoint_1.checkpoint('snapshot releases not supported for this releaser', checkpoint_1.CheckpointType.Failure);
            return;
        }
        const mergedPR = await this.gh.findMergedReleasePR(this.labels);
        if (mergedPR) {
            // a PR already exists in the autorelease: pending state.
            checkpoint_1.checkpoint(`pull #${mergedPR.number} ${mergedPR.sha} has not yet been released`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        else {
            return this._run();
        }
    }
    async _run() {
        throw Error('must be implemented by subclass');
    }
    supportsSnapshots() {
        return false;
    }
    async closeStaleReleasePRs(currentPRNumber, includePackageName = false) {
        const prs = await this.gh.findOpenReleasePRs(this.labels);
        for (let i = 0, pr; i < prs.length; i++) {
            pr = prs[i];
            // don't close the most up-to-date release PR.
            if (pr.number !== currentPRNumber) {
                // on mono repos that maintain multiple open release PRs, we use the
                // pull request title to differentiate between PRs:
                if (includePackageName && !pr.title.includes(` ${this.packageName} `)) {
                    continue;
                }
                checkpoint_1.checkpoint(`closing pull #${pr.number} on ${this.repoUrl}`, checkpoint_1.CheckpointType.Failure);
                await this.gh.closePR(pr.number);
            }
        }
    }
    defaultInitialVersion() {
        return '1.0.0';
    }
    // A releaser can implement this method to automatically detect
    // the release name when creating a GitHub release, for instance by returning
    // name in package.json, or setup.py.
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static async lookupPackageName(gh) {
        return Promise.resolve(undefined);
    }
    static tagSeparator() {
        return '-';
    }
    async coerceReleaseCandidate(cc, latestTag, preRelease = false) {
        var _a, _b;
        const releaseAsRe = /release-as:\s*v?([0-9]+\.[0-9]+\.[0-9a-z]+(-[0-9a-z.]+)?)\s*/i;
        const previousTag = latestTag ? latestTag.name : undefined;
        let version = latestTag ? latestTag.version : this.defaultInitialVersion();
        // If a commit contains the footer release-as: 1.x.x, we use this version
        // from the commit footer rather than the version returned by suggestBump().
        const releaseAsCommit = cc.commits.find((element) => {
            if (element.message.match(releaseAsRe)) {
                return true;
            }
            else {
                return false;
            }
        });
        if (releaseAsCommit) {
            const match = releaseAsCommit.message.match(releaseAsRe);
            version = match[1];
        }
        else if (preRelease) {
            // Handle pre-release format v1.0.0-alpha1, alpha2, etc.
            const [prefix, suffix] = version.split('-');
            const match = suffix === null || suffix === void 0 ? void 0 : suffix.match(/(?<type>[^0-9]+)(?<number>[0-9]+)/);
            const number = Number(((_a = match === null || match === void 0 ? void 0 : match.groups) === null || _a === void 0 ? void 0 : _a.number) || 0) + 1;
            version = `${prefix}-${((_b = match === null || match === void 0 ? void 0 : match.groups) === null || _b === void 0 ? void 0 : _b.type) || 'alpha'}${number}`;
        }
        else if (latestTag && !this.releaseAs) {
            const bump = await cc.suggestBump(version);
            const candidate = semver.inc(version, bump.releaseType);
            if (!candidate)
                throw Error(`failed to increment ${version}`);
            version = candidate;
        }
        else if (this.releaseAs) {
            version = this.releaseAs;
        }
        return { version, previousTag };
    }
    async commits(opts) {
        const sha = opts.sha;
        const perPage = opts.perPage || 100;
        const labels = opts.labels || false;
        const path = opts.path || undefined;
        const commits = await this.gh.commitsSinceSha(sha, perPage, labels, path);
        if (commits.length) {
            checkpoint_1.checkpoint(`found ${commits.length} commits since ${sha ? sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Success);
        }
        else {
            checkpoint_1.checkpoint(`no commits found since ${sha}`, checkpoint_1.CheckpointType.Failure);
        }
        return commits;
    }
    gitHubInstance(octokitAPIs) {
        const [owner, repo] = parseGithubRepoUrl(this.repoUrl);
        return new github_1.GitHub({
            token: this.token,
            defaultBranch: this.defaultBranch,
            owner,
            repo,
            apiUrl: this.apiUrl,
            proxyKey: this.proxyKey,
            octokitAPIs,
        });
    }
    async openPR(options) {
        const sha = options.sha;
        const changelogEntry = options.changelogEntry;
        const updates = options.updates;
        const version = options.version;
        const includePackageName = options.includePackageName;
        // Do not include npm style @org/ prefixes in the branch name:
        const branchPrefix = this.packageName.match(/^@[\w-]+\//)
            ? this.packageName.split('/')[1]
            : this.packageName;
        const title = includePackageName
            ? `chore: release ${this.packageName} ${version}`
            : `chore: release ${version}`;
        const body = `:robot: I have created a release \\*beep\\* \\*boop\\* \n---\n${changelogEntry}\n\nThis PR was generated with [Release Please](https://github.com/googleapis/release-please).`;
        const pr = await this.gh.openPR({
            branch: includePackageName
                ? `release-${branchPrefix}-v${version}`
                : `release-v${version}`,
            version,
            sha,
            updates,
            title,
            body,
            fork: this.fork,
            labels: this.labels,
        });
        // a return of undefined indicates that PR was not updated.
        if (pr) {
            // If the PR is being created from a fork, it will not have permission
            // to add nd remove labels from the PR:
            if (!this.fork) {
                await this.gh.addLabels(this.labels, pr);
            }
            else {
                checkpoint_1.checkpoint('release labels were not added, due to PR being created from fork', checkpoint_1.CheckpointType.Failure);
            }
            checkpoint_1.checkpoint(`${this.repoUrl} find stale PRs with label "${this.labels.join(',')}"`, checkpoint_1.CheckpointType.Success);
            if (!this.fork) {
                await this.closeStaleReleasePRs(pr, includePackageName);
            }
        }
        return pr;
    }
    changelogEmpty(changelogEntry) {
        return changelogEntry.split('\n').length === 1;
    }
    addPath(file) {
        if (this.path === undefined) {
            return file;
        }
        else {
            const path = this.path.replace(/[/\\]$/, '');
            file = file.replace(/^[/\\]/, '');
            return `${path}/${file}`;
        }
    }
}
exports.ReleasePR = ReleasePR;
ReleasePR.releaserName = 'base';
//# sourceMappingURL=release-pr.js.map