"use strict";
// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.GoYoshiSubmodule = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
const changelog_1 = require("../updaters/changelog");
const SCOPE_REGEX = /^\w+\((?<scope>.*)\):/;
class GoYoshiSubmodule extends release_pr_1.ReleasePR {
    async _run() {
        if (!this.packageName) {
            throw Error('GoYoshiSubmodule requires this.packageName');
        }
        // Get tag relative to module/v1.0.0:
        const latestTag = await this.gh.latestTag(`${this.packageName}/`, false, `${this.packageName}`);
        const commits = (await this.commits({
            sha: latestTag === null || latestTag === void 0 ? void 0 : latestTag.sha,
            path: this.path,
        })).filter(commit => {
            var _a, _b;
            const scope = (_b = (_a = commit.message.match(SCOPE_REGEX)) === null || _a === void 0 ? void 0 : _a.groups) === null || _b === void 0 ? void 0 : _b.scope;
            // Filter commits that don't have a scope as we don't know where to put
            // them.
            if (!scope) {
                return false;
            }
            // Only use commits that match our scope:
            if (scope === this.packageName ||
                scope.startsWith(this.packageName + '/')) {
                return true;
            }
            return false;
        });
        const cc = new conventional_commits_1.ConventionalCommits({
            commits,
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
            changelogSections: this.changelogSections,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        const changelogEntry = await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        });
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry)) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return;
        }
        const updates = [];
        updates.push(new changelog_1.Changelog({
            path: this.addPath('CHANGES.md'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        return this.openPR({
            sha: commits[0].sha,
            changelogEntry: `${changelogEntry}\n---\n`,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
}
exports.GoYoshiSubmodule = GoYoshiSubmodule;
GoYoshiSubmodule.releaserName = 'go-yoshi-submodule';
//# sourceMappingURL=go-yoshi-submodule.js.map