"use strict";
// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.GoYoshi = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
const semver = require("semver");
// Generic
const changelog_1 = require("../updaters/changelog");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const parseGithubRepoUrl = require('parse-github-repo-url');
// Commits containing a scope prefixed with an item in this array will be
// ignored when generating a release PR for the parent module.
const SUB_MODULES = [
    'bigtable',
    'bigquery',
    'datastore',
    'firestore',
    'logging',
    'pubsub',
    'pubsublite',
    'spanner',
    'storage',
];
const REGEN_PR_REGEX = /.*auto-regenerate.*/;
const SCOPE_REGEX = /^\w+\((?<scope>.*)\):/;
class GoYoshi extends release_pr_1.ReleasePR {
    async _run() {
        const latestTag = await this.gh.latestTag(this.monorepoTags ? `${this.packageName}/` : undefined, false, this.monorepoTags ? `${this.packageName}` : undefined);
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [_owner, repo] = parseGithubRepoUrl(this.repoUrl);
        let regenPR;
        let sha = null;
        const commits = (await this.commits({
            sha: latestTag === null || latestTag === void 0 ? void 0 : latestTag.sha,
            path: this.path,
        })).filter(commit => {
            var _a, _b, _c, _d;
            if (this.isGapicRepo(repo)) {
                const scope = (_b = (_a = commit.message.match(SCOPE_REGEX)) === null || _a === void 0 ? void 0 : _a.groups) === null || _b === void 0 ? void 0 : _b.scope;
                // Filter commits that don't have a scope as we don't know where to put
                // them.
                if (!scope) {
                    return false;
                }
                // Skipping commits related to sub-modules as they are not apart of the
                // parent module.
                if (!this.monorepoTags) {
                    for (const subModule of SUB_MODULES) {
                        if (scope === subModule || scope.startsWith(subModule + '/')) {
                            return false;
                        }
                    }
                }
                // In the gapic repo, it's required that commits are prefixed with
                // the packageName scope, in order for the commit to count towards
                // a release:
                if (this.monorepoTags) {
                    if (scope === this.packageName ||
                        scope.startsWith(this.packageName + '/')) {
                        return true;
                    }
                    return false;
                }
            }
            // Store the very first SHA returned, this represents the HEAD of the
            // release being created:
            if (!sha) {
                sha = commit.sha;
            }
            if (this.isMultiClientRepo(repo) && REGEN_PR_REGEX.test(commit.message)) {
                // Only have a single entry of the nightly regen listed in the changelog.
                // If there are more than one of these commits, append associated PR.
                const issueRe = /(?<prefix>.*)\((?<pr>.*)\)(\n|$)/;
                if (regenPR) {
                    const match = commit.message.match(issueRe);
                    if ((_c = match === null || match === void 0 ? void 0 : match.groups) === null || _c === void 0 ? void 0 : _c.pr) {
                        regenPR.message += `\nRefs ${match.groups.pr}`;
                    }
                    return false;
                }
                else {
                    // Throw away the sha for nightly regens, will just append PR numbers.
                    commit.sha = null;
                    regenPR = commit;
                    const match = commit.message.match(issueRe);
                    if ((_d = match === null || match === void 0 ? void 0 : match.groups) === null || _d === void 0 ? void 0 : _d.pr) {
                        regenPR.message = `${match.groups.prefix}\n\nRefs ${match.groups.pr}`;
                    }
                }
            }
            return true;
        });
        const cc = new conventional_commits_1.ConventionalCommits({
            commits: commits,
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        // "closes" is a little presumptuous, let's just indicate that the
        // PR references these other commits:
        const changelogEntry = (await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        })).replace(/, closes /g, ', refs ');
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry)) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const updates = [];
        updates.push(new changelog_1.Changelog({
            path: this.addPath('CHANGES.md'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        if (!sha) {
            throw Error('no sha found for pull request');
        }
        return await this.openPR({
            sha: sha,
            changelogEntry,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
    isGapicRepo(repo) {
        return repo === 'google-cloud-go';
    }
    isMultiClientRepo(repo) {
        return repo === 'google-cloud-go' || repo === 'google-api-go-client';
    }
    async coerceReleaseCandidate(cc, latestTag) {
        const version = latestTag
            ? latestTag.version
            : this.defaultInitialVersion();
        const previousTag = latestTag ? latestTag.name : undefined;
        const bump = 'minor';
        const candidate = semver.inc(version, bump);
        return { version: candidate, previousTag };
    }
    defaultInitialVersion() {
        return '0.1.0';
    }
    static tagSeparator() {
        return '/';
    }
}
exports.GoYoshi = GoYoshi;
GoYoshi.releaserName = 'go-yoshi';
//# sourceMappingURL=go-yoshi.js.map