"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaAuthYoshi = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
// Generic
const changelog_1 = require("../updaters/changelog");
// Java
const pom_xml_1 = require("../updaters/java/pom-xml");
// Yoshi Java Auth Library
const versions_manifest_1 = require("../updaters/java/versions-manifest");
const readme_1 = require("../updaters/java/readme");
class JavaAuthYoshi extends release_pr_1.ReleasePR {
    async _run() {
        const latestTag = await this.gh.latestTag();
        const commits = this.snapshot
            ? [
                {
                    sha: 'abc123',
                    message: 'fix: ',
                    files: [],
                },
            ]
            : await this.commits({
                sha: latestTag ? latestTag.sha : undefined,
                labels: true,
            });
        if (commits.length === 0) {
            checkpoint_1.checkpoint(`no commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        let prSHA = commits[0].sha;
        const cc = new conventional_commits_1.ConventionalCommits({
            commits,
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        let changelogEntry = await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        });
        // snapshot entries are special:
        // 1. they don't update the README or CHANGELOG.
        // 2. they always update a patch with the -SNAPSHOT suffix.
        // 3. they're haunted.
        if (this.snapshot) {
            prSHA = latestTag.sha;
            candidate.version = `${candidate.version}-SNAPSHOT`;
            changelogEntry =
                '### Updating meta-information for bleeding-edge SNAPSHOT release.';
        }
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry) && !this.snapshot) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const updates = [];
        if (!this.snapshot) {
            updates.push(new changelog_1.Changelog({
                path: 'CHANGELOG.md',
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName,
            }));
            updates.push(new readme_1.Readme({
                path: 'README.md',
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName,
            }));
        }
        updates.push(new versions_manifest_1.VersionsManifest({
            path: 'versions.txt',
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        [
            'appengine/pom.xml',
            'bom/pom.xml',
            'credentials/pom.xml',
            'oauth2_http/pom.xml',
            'pom.xml',
        ].forEach(path => {
            updates.push(new pom_xml_1.PomXML({
                path,
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName,
            }));
        });
        return await this.openPR({
            sha: prSHA,
            changelogEntry: `${changelogEntry}\n---\n`,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
    supportsSnapshots() {
        return true;
    }
}
exports.JavaAuthYoshi = JavaAuthYoshi;
JavaAuthYoshi.releaserName = 'java-auth-yoshi';
//# sourceMappingURL=java-auth-yoshi.js.map