"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaYoshi = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
// Generic
const changelog_1 = require("../updaters/changelog");
// Java
const google_utils_1 = require("../updaters/java/google-utils");
const pom_xml_1 = require("../updaters/java/pom-xml");
const versions_manifest_1 = require("../updaters/java/versions-manifest");
const readme_1 = require("../updaters/java/readme");
const version_1 = require("./java/version");
const bump_type_1 = require("./java/bump_type");
const java_update_1 = require("../updaters/java/java_update");
const stability_1 = require("./java/stability");
const CHANGELOG_SECTIONS = [
    { type: 'feat', section: 'Features' },
    { type: 'fix', section: 'Bug Fixes' },
    { type: 'perf', section: 'Performance Improvements' },
    { type: 'deps', section: 'Dependencies' },
    { type: 'revert', section: 'Reverts' },
    { type: 'docs', section: 'Documentation' },
    { type: 'style', section: 'Styles', hidden: true },
    { type: 'chore', section: 'Miscellaneous Chores', hidden: true },
    { type: 'refactor', section: 'Code Refactoring', hidden: true },
    { type: 'test', section: 'Tests', hidden: true },
    { type: 'build', section: 'Build System', hidden: true },
    { type: 'ci', section: 'Continuous Integration', hidden: true },
];
class JavaYoshi extends release_pr_1.ReleasePR {
    async _run() {
        const versionsManifestContent = await this.gh.getFileContents('versions.txt');
        const currentVersions = versions_manifest_1.VersionsManifest.parseVersions(versionsManifestContent.parsedContent);
        const snapshotNeeded = versions_manifest_1.VersionsManifest.needsSnapshot(versionsManifestContent.parsedContent);
        if (!this.snapshot) {
            // if a snapshot is not explicitly requested, decided what type
            // of release based on whether a snapshot is needed or not
            this.snapshot = snapshotNeeded;
        }
        else if (!snapshotNeeded) {
            checkpoint_1.checkpoint('release asked for a snapshot, but no snapshot is needed', checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        if (this.snapshot) {
            this.labels = ['type: process'];
        }
        const latestTag = await this.gh.latestTag();
        const commits = this.snapshot
            ? [
                {
                    sha: 'abc123',
                    message: 'fix: ',
                    files: [],
                },
            ]
            : await this.commits({
                sha: latestTag ? latestTag.sha : undefined,
                labels: true,
            });
        if (commits.length === 0) {
            checkpoint_1.checkpoint(`no commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        let prSHA = commits[0].sha;
        // Snapshots populate a fake "fix:"" commit, so that they will always
        // result in a patch update. We still need to know the HEAD sba, so that
        // we can use this as a starting point for the snapshot PR:
        if (this.snapshot && (latestTag === null || latestTag === void 0 ? void 0 : latestTag.sha)) {
            const latestCommit = (await this.commits({
                sha: latestTag.sha,
                perPage: 1,
                labels: true,
            }))[0];
            prSHA = latestCommit ? latestCommit.sha : latestTag.sha;
        }
        const cc = new conventional_commits_1.ConventionalCommits({
            commits,
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
            changelogSections: CHANGELOG_SECTIONS,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        const candidateVersions = await this.coerceVersions(cc, currentVersions, candidate);
        let changelogEntry = await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        });
        // snapshot entries are special:
        // 1. they don't update the README or CHANGELOG.
        // 2. they always update a patch with the -SNAPSHOT suffix.
        // 3. they're haunted.
        if (this.snapshot) {
            candidate.version = `${candidate.version}-SNAPSHOT`;
            changelogEntry =
                '### Updating meta-information for bleeding-edge SNAPSHOT release.';
        }
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry) && !this.snapshot) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const updates = [];
        if (!this.snapshot) {
            updates.push(new changelog_1.Changelog({
                path: 'CHANGELOG.md',
                changelogEntry,
                versions: candidateVersions,
                version: candidate.version,
                packageName: this.packageName,
            }));
            updates.push(new readme_1.Readme({
                path: 'README.md',
                changelogEntry,
                versions: candidateVersions,
                version: candidate.version,
                packageName: this.packageName,
            }));
            updates.push(new google_utils_1.GoogleUtils({
                // TODO(@chingor): should this use search like pom.xml?
                path: 'google-api-client/src/main/java/com/google/api/client/googleapis/GoogleUtils.java',
                changelogEntry,
                versions: candidateVersions,
                version: candidate.version,
                packageName: this.packageName,
                contents: versionsManifestContent,
            }));
        }
        updates.push(new versions_manifest_1.VersionsManifest({
            path: 'versions.txt',
            changelogEntry,
            versions: candidateVersions,
            version: candidate.version,
            packageName: this.packageName,
            contents: versionsManifestContent,
        }));
        const pomFilesSearch = this.gh.findFilesByFilename('pom.xml');
        const buildFilesSearch = this.gh.findFilesByFilename('build.gradle');
        const dependenciesSearch = this.gh.findFilesByFilename('dependencies.properties');
        const pomFiles = await pomFilesSearch;
        pomFiles.forEach(path => {
            updates.push(new pom_xml_1.PomXML({
                path,
                changelogEntry,
                versions: candidateVersions,
                version: candidate.version,
                packageName: this.packageName,
            }));
        });
        const buildFiles = await buildFilesSearch;
        buildFiles.forEach(path => {
            updates.push(new java_update_1.JavaUpdate({
                path,
                changelogEntry,
                versions: candidateVersions,
                version: candidate.version,
                packageName: this.packageName,
            }));
        });
        const dependenciesFiles = await dependenciesSearch;
        dependenciesFiles.forEach(path => {
            updates.push(new java_update_1.JavaUpdate({
                path,
                changelogEntry,
                versions: candidateVersions,
                version: candidate.version,
                packageName: this.packageName,
            }));
        });
        return await this.openPR({
            sha: prSHA,
            changelogEntry: `${changelogEntry}\n---\n`,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
    supportsSnapshots() {
        return true;
    }
    defaultInitialVersion() {
        return '0.1.0';
    }
    async coerceVersions(cc, currentVersions, candidate) {
        const newVersions = new Map();
        for (const [k, version] of currentVersions) {
            if (candidate.version === '1.0.0' && stability_1.isStableArtifact(k)) {
                newVersions.set(k, '1.0.0');
            }
            else {
                const bump = await cc.suggestBump(version);
                const newVersion = version_1.Version.parse(version);
                newVersion.bump(this.snapshot ? 'snapshot' : bump_type_1.fromSemverReleaseType(bump.releaseType));
                newVersions.set(k, newVersion.toString());
            }
        }
        return newVersions;
    }
}
exports.JavaYoshi = JavaYoshi;
JavaYoshi.releaserName = 'java-yoshi';
//# sourceMappingURL=java-yoshi.js.map