"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Node = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
// Generic
const changelog_1 = require("../updaters/changelog");
// JavaScript
const package_json_1 = require("../updaters/package-json");
const samples_package_json_1 = require("../updaters/samples-package-json");
class Node extends release_pr_1.ReleasePR {
    async _run() {
        const latestTag = await this.gh.latestTag(this.monorepoTags ? `${this.packageName}-` : undefined);
        const commits = await this.commits({
            sha: latestTag ? latestTag.sha : undefined,
            path: this.path,
        });
        const cc = new conventional_commits_1.ConventionalCommits({
            commits,
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
            changelogSections: this.changelogSections,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        const changelogEntry = await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        });
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry)) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const updates = [];
        // Make an effort to populate packageName from the contents of
        // the package.json, rather than forcing this to be set:
        const contents = await this.gh.getFileContents(this.addPath('package.json'));
        const pkg = JSON.parse(contents.parsedContent);
        if (pkg.name)
            this.packageName = pkg.name;
        updates.push(new package_json_1.PackageJson({
            path: this.addPath('package-lock.json'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        updates.push(new samples_package_json_1.SamplesPackageJson({
            path: this.addPath('samples/package.json'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        updates.push(new changelog_1.Changelog({
            path: this.addPath('CHANGELOG.md'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        updates.push(new package_json_1.PackageJson({
            path: this.addPath('package.json'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
            contents,
        }));
        return await this.openPR({
            sha: commits[0].sha,
            changelogEntry: `${changelogEntry}\n---\n`,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
    // A releaser can implement this method to automatically detect
    // the release name when creating a GitHub release, for instance by returning
    // name in package.json, or setup.py.
    static async lookupPackageName(gh) {
        // Make an effort to populate packageName from the contents of
        // the package.json, rather than forcing this to be set:
        const contents = await gh.getFileContents('package.json');
        const pkg = JSON.parse(contents.parsedContent);
        if (pkg.name)
            return pkg.name;
        else
            return undefined;
    }
}
exports.Node = Node;
Node.releaserName = 'node';
//# sourceMappingURL=node.js.map