"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Python = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
// Generic
const changelog_1 = require("../updaters/changelog");
// Python specific.
const setup_py_1 = require("../updaters/python/setup-py");
const setup_cfg_1 = require("../updaters/python/setup-cfg");
const version_py_1 = require("../updaters/python/version-py");
const CHANGELOG_SECTIONS = [
    { type: 'feat', section: 'Features' },
    { type: 'fix', section: 'Bug Fixes' },
    { type: 'perf', section: 'Performance Improvements' },
    { type: 'deps', section: 'Dependencies' },
    { type: 'revert', section: 'Reverts' },
    { type: 'docs', section: 'Documentation' },
    { type: 'style', section: 'Styles', hidden: true },
    { type: 'chore', section: 'Miscellaneous Chores', hidden: true },
    { type: 'refactor', section: 'Code Refactoring', hidden: true },
    { type: 'test', section: 'Tests', hidden: true },
    { type: 'build', section: 'Build System', hidden: true },
    { type: 'ci', section: 'Continuous Integration', hidden: true },
];
class Python extends release_pr_1.ReleasePR {
    async _run() {
        const latestTag = await this.gh.latestTag(this.monorepoTags ? `${this.packageName}-` : undefined);
        const commits = await this.commits({
            sha: latestTag ? latestTag.sha : undefined,
            path: this.path,
        });
        const cc = new conventional_commits_1.ConventionalCommits({
            commits,
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
            changelogSections: this.changelogSections || CHANGELOG_SECTIONS,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        const changelogEntry = await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        });
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry)) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const updates = [];
        updates.push(new changelog_1.Changelog({
            path: this.addPath('CHANGELOG.md'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        updates.push(new setup_cfg_1.SetupCfg({
            path: this.addPath('setup.cfg'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        updates.push(new setup_py_1.SetupPy({
            path: this.addPath('setup.py'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        // There should be only one version.py, but foreach in case that is incorrect
        const versionPyFilesSearch = this.gh.findFilesByFilename('version.py');
        const versionPyFiles = await versionPyFilesSearch;
        versionPyFiles.forEach(path => {
            updates.push(new version_py_1.VersionPy({
                path: this.addPath(path),
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName,
            }));
        });
        return await this.openPR({
            sha: commits[0].sha,
            changelogEntry: `${changelogEntry}\n---\n`,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
    defaultInitialVersion() {
        return '0.1.0';
    }
}
exports.Python = Python;
Python.releaserName = 'python';
//# sourceMappingURL=python.js.map