"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ruby = void 0;
const release_pr_1 = require("../release-pr");
const conventional_commits_1 = require("../conventional-commits");
const checkpoint_1 = require("../util/checkpoint");
const indent_commit_1 = require("../util/indent-commit");
// Generic
const changelog_1 = require("../updaters/changelog");
// Ruby
const version_rb_1 = require("../updaters/version-rb");
class Ruby extends release_pr_1.ReleasePR {
    constructor(options) {
        super(options);
        this.versionFile = options.versionFile;
    }
    async _run() {
        const latestTag = await this.gh.latestTag(this.monorepoTags ? `${this.packageName}/` : undefined, false, this.monorepoTags ? `${this.packageName}-` : undefined);
        const commits = await this.commits({
            sha: latestTag ? latestTag.sha : undefined,
            path: this.path,
        });
        const cc = new conventional_commits_1.ConventionalCommits({
            commits: postProcessCommits(commits),
            githubRepoUrl: this.repoUrl,
            bumpMinorPreMajor: this.bumpMinorPreMajor,
            changelogSections: this.changelogSections,
        });
        const candidate = await this.coerceReleaseCandidate(cc, latestTag);
        const changelogEntry = await cc.generateChangelogEntry({
            version: candidate.version,
            currentTag: `v${candidate.version}`,
            previousTag: candidate.previousTag,
        });
        // don't create a release candidate until user facing changes
        // (fix, feat, BREAKING CHANGE) have been made; a CHANGELOG that's
        // one line is a good indicator that there were no interesting commits.
        if (this.changelogEmpty(changelogEntry)) {
            checkpoint_1.checkpoint(`no user facing commits found since ${latestTag ? latestTag.sha : 'beginning of time'}`, checkpoint_1.CheckpointType.Failure);
            return undefined;
        }
        const updates = [];
        updates.push(new changelog_1.Changelog({
            path: this.addPath('CHANGELOG.md'),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        updates.push(new version_rb_1.VersionRB({
            path: this.addPath(this.versionFile),
            changelogEntry,
            version: candidate.version,
            packageName: this.packageName,
        }));
        return await this.openPR({
            sha: commits[0].sha,
            changelogEntry: `${changelogEntry}\n---\n`,
            updates,
            version: candidate.version,
            includePackageName: this.monorepoTags,
        });
    }
    static tagSeparator() {
        return '/';
    }
}
exports.Ruby = Ruby;
Ruby.releaserName = 'ruby';
function postProcessCommits(commits) {
    commits.forEach(commit => {
        commit.message = indent_commit_1.indentCommit(commit);
    });
    return commits;
}
//# sourceMappingURL=ruby.js.map